module xillybus_block
  (
  input  PCIE_PERST_B_LS,
  input  PCIE_REFCLK_N,
  input  PCIE_REFCLK_P,
  input [31:0] axi_data_r_read_32,
  input [7:0] PCIE_RX_N,
  input [7:0] PCIE_RX_P,
  input [7:0] axi_data_r_read_8,
  input ap_clk,
  input axi_ready_w_write_32,
  input axi_ready_w_write_8,
  input axi_valid_r_read_32,
  input axi_valid_r_read_8,
  output  bus_clk,
  output  quiesce,
  output [31:0] axi_data_w_write_32,
  output [3:0] GPIO_LED,
  output [7:0] PCIE_TX_N,
  output [7:0] PCIE_TX_P,
  output [7:0] axi_data_w_write_8,
  output axi_ready_r_read_32,
  output axi_ready_r_read_8,
  output axi_valid_w_write_32,
  output axi_valid_w_write_8,
  output reg from_host_write_32_open,
  output reg from_host_write_8_open,
  output reg to_host_read_32_open,
  output reg to_host_read_8_open
);
  wire  trn_reset_n;
  wire  trn_lnk_up_n;
  wire [3:0] s_axis_rq_tready;
  wire [63:0] s_axis_rq_tdata;
  wire [1:0] s_axis_rq_tkeep;
  wire  s_axis_rq_tlast;
  wire  s_axis_rq_tvalid;
  wire [59:0] s_axis_rq_tuser;
  wire [63:0] m_axis_rc_tdata;
  wire [1:0] m_axis_rc_tkeep;
  wire  m_axis_rc_tlast;
  wire  m_axis_rc_tvalid;
  wire [21:0] m_axis_rc_tready;
  wire [63:0] m_axis_cq_tdata;
  wire [1:0] m_axis_cq_tkeep;
  wire  m_axis_cq_tlast;
  wire  m_axis_cq_tvalid;
  wire [21:0] m_axis_cq_tready;
  wire [31:0] cfg_interrupt_msi_int;
  wire [63:0] cfg_interrupt_msi_pending_status;
  wire  cfg_interrupt_msi_sent;
  wire  cfg_interrupt_msi_fail;
  wire [7:0] trn_fc_cplh;
  wire [11:0] trn_fc_cpld;
  wire  trn_terr_drop_n;
  wire [3:0] pcie_rq_seq_num;
  wire  pcie_rq_seq_num_vld;
  wire [2:0] cfg_max_payload;
  wire [2:0] cfg_max_read_req;
  wire [1:0] cfg_rcb_status;
  wire  cfg_err_fatal_out;
  wire  pcie_ref_clk;
  wire  pcie_ref_clk_gt;
  wire  user_reset_out;
  wire  user_lnk_up;

  // Wires related to /dev/xillybus_mem_8
  wire  user_r_mem_8_rden_w;
  wire  user_r_mem_8_empty_w;
  wire [7:0] user_r_mem_8_data_w;
  wire  user_r_mem_8_eof_w;
  wire  user_r_mem_8_open_w;
  wire  user_w_mem_8_wren_w;
  wire  user_w_mem_8_full_w;
  wire [7:0] user_w_mem_8_data_w;
  wire  user_w_mem_8_open_w;
  wire [4:0] user_mem_8_addr_w;
  wire  user_mem_8_addr_update_w;
  reg user_r_mem_8_rden_d;
  reg user_w_mem_8_wren_d;

  // Wires related to /dev/xillybus_read_32
  wire  user_r_read_32_rden_w;
  wire  user_r_read_32_empty_w;
  wire [31:0] user_r_read_32_data_w;
  wire  user_r_read_32_eof_w;
  wire  user_r_read_32_open_w;
  wire full_r_read_32;
  reg to_host_read_32_open_pre1;
  reg to_host_read_32_open_pre2;

  // Wires related to /dev/xillybus_read_8
  wire  user_r_read_8_rden_w;
  wire  user_r_read_8_empty_w;
  wire [7:0] user_r_read_8_data_w;
  wire  user_r_read_8_eof_w;
  wire  user_r_read_8_open_w;
  wire full_r_read_8;
  reg to_host_read_8_open_pre1;
  reg to_host_read_8_open_pre2;

  // Wires related to /dev/xillybus_write_32
  wire  user_w_write_32_wren_w;
  wire  user_w_write_32_full_w;
  wire [31:0] user_w_write_32_data_w;
  wire  user_w_write_32_open_w;
  wire empty_w_write_32;
  reg from_host_write_32_open_pre1;
  reg from_host_write_32_open_pre2;

  // Wires related to /dev/xillybus_write_8
  wire  user_w_write_8_wren_w;
  wire  user_w_write_8_full_w;
  wire [7:0] user_w_write_8_data_w;
  wire  user_w_write_8_open_w;
  wire empty_w_write_8;
  reg from_host_write_8_open_pre1;
  reg from_host_write_8_open_pre2;

   // Bogus connections of some user_r_mem_8_* signals to prevent trimming
   // of logic which would result in Vivado failing to implement the design.
   // These assignments have no functional purpose.

   assign user_r_mem_8_eof_w = 1;
   assign user_r_mem_8_empty_w = user_r_mem_8_rden_d || user_r_mem_8_open_w;
   assign user_r_mem_8_data_w = 0;

   always @(posedge bus_clk)
     user_r_mem_8_rden_d <= user_r_mem_8_rden_w;

   // Bogus connections of some user_w_mem_8_* signals to prevent trimming
   // of logic which would result in Vivado failing to implement the design.
   // This assignment has no functional purpose.

   assign user_w_mem_8_full_w = user_w_mem_8_open_w || user_w_mem_8_wren_d;

   always @(posedge bus_clk)
     user_w_mem_8_wren_d <= user_w_mem_8_wren_w;

   fifo_xillybus_32 r_read_32
     (
      .rst(!user_r_read_32_open_w),
      .wr_clk(ap_clk),
      .rd_clk(bus_clk),
      .din(axi_data_r_read_32),
      .wr_en(axi_ready_r_read_32 && axi_valid_r_read_32),
      .rd_en(user_r_read_32_rden_w),
      .dout(user_r_read_32_data_w),
      .full(full_r_read_32),
      .empty(user_r_read_32_empty_w)
      );

   assign user_r_read_32_eof_w = 0;
   assign axi_ready_r_read_32 = !full_r_read_32;

   always @(posedge ap_clk)
     begin
       to_host_read_32_open_pre1 <= user_r_read_32_open_w;
       to_host_read_32_open_pre2 <= to_host_read_32_open_pre1;
       to_host_read_32_open <= to_host_read_32_open_pre2;
     end

   fifo_xillybus_8 r_read_8
     (
      .rst(!user_r_read_8_open_w),
      .wr_clk(ap_clk),
      .rd_clk(bus_clk),
      .din(axi_data_r_read_8),
      .wr_en(axi_ready_r_read_8 && axi_valid_r_read_8),
      .rd_en(user_r_read_8_rden_w),
      .dout(user_r_read_8_data_w),
      .full(full_r_read_8),
      .empty(user_r_read_8_empty_w)
      );

   assign user_r_read_8_eof_w = 0;
   assign axi_ready_r_read_8 = !full_r_read_8;

   always @(posedge ap_clk)
     begin
       to_host_read_8_open_pre1 <= user_r_read_8_open_w;
       to_host_read_8_open_pre2 <= to_host_read_8_open_pre1;
       to_host_read_8_open <= to_host_read_8_open_pre2;
     end

   fifo_xillybus_fwft_32 fifo_w_write_32
     (
      .rst(!user_w_write_32_open_w),
      .wr_clk(bus_clk),
      .rd_clk(ap_clk),
      .din(user_w_write_32_data_w),
      .wr_en(user_w_write_32_wren_w),
      .rd_en(axi_ready_w_write_32 && axi_valid_w_write_32),
      .dout(axi_data_w_write_32),
      .full(user_w_write_32_full_w),
      .empty(empty_w_write_32)
      );

   assign axi_valid_w_write_32 = !empty_w_write_32;

   always @(posedge ap_clk)
     begin
       from_host_write_32_open_pre1 <= user_w_write_32_open_w;
       from_host_write_32_open_pre2 <= from_host_write_32_open_pre1;
       from_host_write_32_open <= from_host_write_32_open_pre2;
     end

   fifo_xillybus_fwft_8 fifo_w_write_8
     (
      .rst(!user_w_write_8_open_w),
      .wr_clk(bus_clk),
      .rd_clk(ap_clk),
      .din(user_w_write_8_data_w),
      .wr_en(user_w_write_8_wren_w),
      .rd_en(axi_ready_w_write_8 && axi_valid_w_write_8),
      .dout(axi_data_w_write_8),
      .full(user_w_write_8_full_w),
      .empty(empty_w_write_8)
      );

   assign axi_valid_w_write_8 = !empty_w_write_8;

   always @(posedge ap_clk)
     begin
       from_host_write_8_open_pre1 <= user_w_write_8_open_w;
       from_host_write_8_open_pre2 <= from_host_write_8_open_pre1;
       from_host_write_8_open <= from_host_write_8_open_pre2;
     end


  xillybus_core xillybus_core_ins (

    // Ports related to /dev/xillybus_mem_8
    // FPGA to CPU signals:
    .user_r_mem_8_rden_w(user_r_mem_8_rden_w),
    .user_r_mem_8_empty_w(user_r_mem_8_empty_w),
    .user_r_mem_8_data_w(user_r_mem_8_data_w),
    .user_r_mem_8_eof_w(user_r_mem_8_eof_w),
    .user_r_mem_8_open_w(user_r_mem_8_open_w),

    // CPU to FPGA signals:
    .user_w_mem_8_wren_w(user_w_mem_8_wren_w),
    .user_w_mem_8_full_w(user_w_mem_8_full_w),
    .user_w_mem_8_data_w(user_w_mem_8_data_w),
    .user_w_mem_8_open_w(user_w_mem_8_open_w),

    // Address signals:
    .user_mem_8_addr_w(user_mem_8_addr_w),
    .user_mem_8_addr_update_w(user_mem_8_addr_update_w),


    // Ports related to /dev/xillybus_read_32
    // FPGA to CPU signals:
    .user_r_read_32_rden_w(user_r_read_32_rden_w),
    .user_r_read_32_empty_w(user_r_read_32_empty_w),
    .user_r_read_32_data_w(user_r_read_32_data_w),
    .user_r_read_32_eof_w(user_r_read_32_eof_w),
    .user_r_read_32_open_w(user_r_read_32_open_w),


    // Ports related to /dev/xillybus_read_8
    // FPGA to CPU signals:
    .user_r_read_8_rden_w(user_r_read_8_rden_w),
    .user_r_read_8_empty_w(user_r_read_8_empty_w),
    .user_r_read_8_data_w(user_r_read_8_data_w),
    .user_r_read_8_eof_w(user_r_read_8_eof_w),
    .user_r_read_8_open_w(user_r_read_8_open_w),


    // Ports related to /dev/xillybus_write_32
    // CPU to FPGA signals:
    .user_w_write_32_wren_w(user_w_write_32_wren_w),
    .user_w_write_32_full_w(user_w_write_32_full_w),
    .user_w_write_32_data_w(user_w_write_32_data_w),
    .user_w_write_32_open_w(user_w_write_32_open_w),


    // Ports related to /dev/xillybus_write_8
    // CPU to FPGA signals:
    .user_w_write_8_wren_w(user_w_write_8_wren_w),
    .user_w_write_8_full_w(user_w_write_8_full_w),
    .user_w_write_8_data_w(user_w_write_8_data_w),
    .user_w_write_8_open_w(user_w_write_8_open_w),


    // General signals
    .trn_reset_n_w(trn_reset_n),
    .trn_lnk_up_n_w(trn_lnk_up_n),
    .s_axis_rq_tready_w(s_axis_rq_tready),
    .m_axis_rc_tdata_w(m_axis_rc_tdata),
    .m_axis_rc_tkeep_w(m_axis_rc_tkeep),
    .m_axis_rc_tlast_w(m_axis_rc_tlast),
    .m_axis_rc_tvalid_w(m_axis_rc_tvalid),
    .m_axis_cq_tdata_w(m_axis_cq_tdata),
    .m_axis_cq_tkeep_w(m_axis_cq_tkeep),
    .m_axis_cq_tlast_w(m_axis_cq_tlast),
    .m_axis_cq_tvalid_w(m_axis_cq_tvalid),
    .cfg_interrupt_msi_sent_w(cfg_interrupt_msi_sent),
    .cfg_interrupt_msi_fail_w(cfg_interrupt_msi_fail),
    .trn_fc_cplh_w(trn_fc_cplh),
    .trn_fc_cpld_w(trn_fc_cpld),
    .trn_terr_drop_n_w(trn_terr_drop_n),
    .pcie_rq_seq_num_w(pcie_rq_seq_num),
    .pcie_rq_seq_num_vld_w(pcie_rq_seq_num_vld),
    .cfg_max_payload_w(cfg_max_payload),
    .cfg_max_read_req_w(cfg_max_read_req),
    .cfg_rcb_status_w(cfg_rcb_status),
    .cfg_err_fatal_out_w(cfg_err_fatal_out),
    .bus_clk_w(bus_clk),
    .s_axis_rq_tdata_w(s_axis_rq_tdata),
    .s_axis_rq_tkeep_w(s_axis_rq_tkeep),
    .s_axis_rq_tlast_w(s_axis_rq_tlast),
    .s_axis_rq_tvalid_w(s_axis_rq_tvalid),
    .s_axis_rq_tuser_w(s_axis_rq_tuser),
    .m_axis_rc_tready_w(m_axis_rc_tready),
    .m_axis_cq_tready_w(m_axis_cq_tready),
    .cfg_interrupt_msi_int_w(cfg_interrupt_msi_int),
    .cfg_interrupt_msi_pending_status_w(cfg_interrupt_msi_pending_status),
    .GPIO_LED_w(GPIO_LED),
    .quiesce_w(quiesce)
  );



      
 
 
   






  
   IBUFDS_GTE3 pcieclk_ibuf (.O(pcie_ref_clk_gt), .ODIV2(pcie_ref_clk),
			     .I(PCIE_REFCLK_P), .IB(PCIE_REFCLK_N),
			     .CEB(1'b0));
   
   assign 	     trn_reset_n = !user_reset_out;
   assign 	     trn_lnk_up_n = !user_lnk_up;
   assign 	     trn_terr_drop_n = 1;

   pcie_ku pcie
     (
      .pci_exp_txp( PCIE_TX_P ),
      .pci_exp_txn( PCIE_TX_N ),
      .pci_exp_rxp( PCIE_RX_P ),
      .pci_exp_rxn( PCIE_RX_N ),

      .user_clk(bus_clk),
      .user_reset(user_reset_out),
      .user_lnk_up(user_lnk_up),
      
      .s_axis_rq_tlast(s_axis_rq_tlast),
      .s_axis_rq_tdata(s_axis_rq_tdata),
      .s_axis_rq_tuser(s_axis_rq_tuser),
      .s_axis_rq_tkeep(s_axis_rq_tkeep),
      .s_axis_rq_tready(s_axis_rq_tready),
      .s_axis_rq_tvalid(s_axis_rq_tvalid),

      .m_axis_rc_tdata(m_axis_rc_tdata),
      .m_axis_rc_tuser(),
      .m_axis_rc_tlast(m_axis_rc_tlast),
      .m_axis_rc_tkeep(m_axis_rc_tkeep),
      .m_axis_rc_tvalid(m_axis_rc_tvalid),
      .m_axis_rc_tready(m_axis_rc_tready),

      .m_axis_cq_tdata(m_axis_cq_tdata),
      .m_axis_cq_tuser(),
      .m_axis_cq_tlast(m_axis_cq_tlast),
      .m_axis_cq_tkeep(m_axis_cq_tkeep),
      .m_axis_cq_tvalid(m_axis_cq_tvalid),
      .m_axis_cq_tready(m_axis_cq_tready),

      .s_axis_cc_tdata(64'd0),
      .s_axis_cc_tuser(33'd0),
      .s_axis_cc_tlast(1'b0),
      .s_axis_cc_tkeep(2'd0),
      .s_axis_cc_tvalid(1'b0),
      .s_axis_cc_tready(),
 
      .pcie_rq_seq_num(pcie_rq_seq_num),
      .pcie_rq_seq_num_vld(pcie_rq_seq_num_vld),
      .pcie_cq_np_req(1'b1),

      .cfg_max_payload(cfg_max_payload),
      .cfg_max_read_req(cfg_max_read_req),
      .cfg_rcb_status(cfg_rcb_status),

      .cfg_err_fatal_out(cfg_err_fatal_out),
 
      .cfg_fc_cpld(trn_fc_cpld), // Completion Data credits
      .cfg_fc_cplh(trn_fc_cplh), // Completion Header credits
      .cfg_fc_sel(3'd0), // Receive credit available space

      .cfg_per_function_number(4'd0),
      .cfg_per_function_output_request(1'b0),

      .cfg_dsn(64'd0),

      .cfg_power_state_change_ack(1'b1),
      .cfg_err_cor_in(1'b0),
      .cfg_err_uncor_in(1'b0),
      .cfg_flr_done(4'd0), // Never recover from Function Level Reset
      .cfg_vf_flr_done(8'd0), // Ditto
      
      .cfg_link_training_enable(1'b1),
      
      .cfg_ds_port_number(8'd0),
            
      .cfg_interrupt_msi_int(cfg_interrupt_msi_int),
      .cfg_interrupt_msi_pending_status(cfg_interrupt_msi_pending_status),
      .cfg_interrupt_msi_sent(cfg_interrupt_msi_sent),
      .cfg_interrupt_msi_fail(cfg_interrupt_msi_fail),

      .cfg_interrupt_msi_attr(3'd0),
      .cfg_interrupt_msi_tph_present(1'b0),
      .cfg_interrupt_msi_tph_type(2'd0),
      .cfg_interrupt_msi_tph_st_tag(9'd0),
      .cfg_interrupt_msi_pending_status_function_num( 4'b0),
      .cfg_interrupt_msi_pending_status_data_enable( 1'b0),

      .cfg_interrupt_msi_function_number(4'd0), // PF0

      .cfg_interrupt_int(4'd0),
      .cfg_interrupt_pending(4'd0),
      .cfg_interrupt_msi_select(4'd0),

      .cfg_config_space_enable(1'b1),
      .cfg_req_pm_transition_l23_ready(1'b0),
      .cfg_hot_reset_in(1'b0),
      .cfg_ds_bus_number(8'd0), // Ignored
      .cfg_ds_device_number(5'd0), // Ignored
      .cfg_ds_function_number(3'd0), // Ignored

      .cfg_subsys_vend_id(16'h10ee),

      .pcie_perstn1_in(1'b0), // Yes, it should be zero

      .sys_clk(pcie_ref_clk),
      .sys_clk_gt(pcie_ref_clk_gt),
      .sys_reset(PCIE_PERST_B_LS)
      );
endmodule
